import 'package:dio/dio.dart';
import 'package:morph_dart/src/models/record/record.dart';

class RecordMethods {
  final Dio _dio;

  const RecordMethods(this._dio);

  Future<RecordListResponse> queryRecords({
    required String tableSlug,
    required QueryRecordRequestBody body,
  }) async {
    final response = await _dio
        .post<Map<String, dynamic>>(
          '/record/query',
          options: Options(
            headers: {
              'tableSlug': tableSlug,
            },
          ),
          data: body,
        )
        .then((res) => RecordListResponse.fromJson(res.data!));

    return response;
  }

  Future<DownloadRecordResponse> downloadRecords({
    required String tableSlug,
    required DownloadRecordRequestBody body,
  }) async {
    final response = await _dio
        .post<Map<String, dynamic>>(
          '/record/download',
          options: Options(
            headers: {
              'tableSlug': tableSlug,
            },
          ),
          data: body,
        )
        .then((res) => DownloadRecordResponse.fromJson(res.data!));

    return response;
  }

  Future<RecordResponse> createRecords({
    required String tableSlug,
    required CreateRecordRequestBody body,
  }) async {
    final response = await _dio
        .post<Map<String, dynamic>>(
          '/record',
          options: Options(
            headers: {
              'tableSlug': tableSlug,
            },
          ),
          data: body,
        )
        .then((res) => RecordResponse.fromJson(res.data!));

    return response;
  }

  Future<RecordResponse> updateRecords({
    required String tableSlug,
    required String recordId,
    required UpdateRecordRequestBody body,
  }) async {
    final response = await _dio
        .put<Map<String, dynamic>>(
          '/record/$recordId',
          options: Options(
            headers: {
              'tableSlug': tableSlug,
            },
          ),
          data: body,
        )
        .then((res) => RecordResponse.fromJson(res.data!));

    return response;
  }

  Future<bool> deleteRecords({
    required String tableSlug,
    required String recordId,
  }) async {
    final response = await _dio
        .delete<Map<String, dynamic>>(
          '/record/$recordId',
          options: Options(
            headers: {
              'tableSlug': tableSlug,
            },
          ),
        )
        .then((res) => res.data!['message'] == 'ok');

    return response;
  }
}
