import 'package:dio/dio.dart';
import 'package:morph_dart/src/enums/table.dart';
import 'package:morph_dart/src/models/table/table.dart';

class TableMethods {
  final Dio _dio;

  const TableMethods(this._dio);

  Future<TableListResponse> getTables({
    num? limit,
    num? skip,
    TableType? tableType,
    String? userpoolTeamId,
    num? linkedTableDepthMax,
    bool? withDeleted,
  }) async {
    final response = await _dio
        .get<Map<String, dynamic>>(
          '/table',
          queryParameters: {
            'limit': limit,
            'skip': skip,
            'tableType': tableType?.name,
            'userpoolTeamId': userpoolTeamId,
            'linkedTableDepthMax': linkedTableDepthMax,
            'withDeleted': withDeleted,
          }..removeWhere((key, value) => value == null),
        )
        .then((res) => TableListResponse.fromJson(res.data!));

    return response;
  }

  Future<TableResponse> findTable({
    required String tableSlug,
    num? linkedTableDepthMax,
  }) async {
    final response = await _dio
        .get<Map<String, dynamic>>(
          '/table/$tableSlug',
          queryParameters: {
            'linkedTableDepthMax': linkedTableDepthMax,
          }..removeWhere((key, value) => value == null),
        )
        .then((res) => TableResponse.fromJson(res.data!));

    return response;
  }

  Future<TableResponse> createTable({
    required CreateTableRequestBody body,
    num? linkedTableDepthMax,
  }) async {
    final response = await _dio
        .post<Map<String, dynamic>>(
          '/table',
          queryParameters: {
            'linkedTableDepthMax': linkedTableDepthMax,
          }..removeWhere((key, value) => value == null),
          data: body,
        )
        .then((res) => TableResponse.fromJson(res.data!));

    return response;
  }

  Future<TableResponse> updateTable({
    required String tableSlug,
    required UpdateTableRequestBody body,
    num? linkedTableDepthMax,
  }) async {
    final response = await _dio
        .put<Map<String, dynamic>>(
          '/table/$tableSlug',
          queryParameters: {
            'linkedTableDepthMax': linkedTableDepthMax,
          }..removeWhere((key, value) => value == null),
          data: body,
        )
        .then((res) => TableResponse.fromJson(res.data!));

    return response;
  }

  Future<bool> deleteTable({
    required String tableSlug,
  }) async {
    final response = await _dio
        .delete<Map<String, dynamic>>('/table/$tableSlug')
        .then((res) => res.data!['message'] == 'ok');

    return response;
  }

  Future<CheckTableSlugResponse> checkTableSlug({
    required String tableSlug,
  }) async {
    final response = await _dio
        .get<Map<String, dynamic>>('/table/$tableSlug/check')
        .then((res) => CheckTableSlugResponse.fromJson(res.data!));

    return response;
  }
}
